%include %{_rpmconfigdir}/macros.python3

# Don't process requires for .py* files, since it massively increases build time
# and we already know that we depend Python -- we're building it!
%define __requires_exclude_from ^%{_libdir}/.*[.]py[oc]?$

%define major 3
%define minor 6
%define fix 15
%define priority %(printf '%02d%02d' %{major} %{minor})

Name: python%{major}
Version: %{major}.%{minor}.%{fix}
Release: 1
License: Python-2.0
Summary: Python 3 Programming Language
Url: https://www.python.org

BuildRequires: openssl-devel >= 1.1.1
BuildRequires: zlib-devel
BuildRequires: expat-devel
BuildRequires: libffi-devel
BuildRequires: xz-devel
BuildRequires: bzip2-devel
BuildRequires: ncurses-devel
BuildRequires: readline-devel
BuildRequires: sqlite3-devel
BuildRequires: tk-devel
BuildRequires: libutil-devel >= 0.7
BuildRequires: pkg-config

# if this is available, _locale will link to it
# Is there a benefit?
# BuildRequires: libintl-devel

Requires: libutil2 >= 0.7
Requires(post): update-alternatives
Requires(postun): update-alternatives

Source0: https://www.python.org/ftp/python/%{version}/Python-%{version}.tar.xz
Source1: python3-sitecustomize.py

Patch0: python3-configure.patch
Patch1: python3-Lib_distutils_util.py.patch
Patch2: python3-Lib_sysconfig.py.patch
Patch3: python3-Modules__ctypes_libffi_configure.patch
Patch4: python3-Python_dynload_aix.c.patch
Patch5: python3-Python_dynload_shlib.c.patch
Patch6: python3-clock_getres.patch
Patch7: python3-dev_t-64bit.patch
Patch8: python3-Python_multiprocessing_semaphore.c.patch
Patch9: python3-link-cpp.patch
Patch10: python3-makexp_aix.patch
Patch11: python3-o_cloexec.patch
Patch12: python3-o_nofollow.patch

%description
Python is a programming language that lets you work more quickly and integrate your systems more effectively.

You can learn to use Python and see almost immediate gains in productivity and lower maintenance costs.

%package devel
Summary: Python 3 development files
Requires: %{name} = %{version}-%{release}

%description devel
Python 3 development files

%package tkinter
Summary: A graphical user interface for the Python 3 language

%description tkinter
The Tkinter (Tk interface) library is a graphical user interface toolkit for
the Python programming language.

%prep
%setup -q -n Python-%{version}

%patch0 -p1
%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1
%if 0%{?ibmi_version} <= 73
%patch6 -p1
%endif
%patch7 -p1
%patch8 -p1
%patch9 -p1
%patch10 -p1
%patch11 -p1
%patch12 -p1

%build

# NOTE: To build with IPV6 support you need the following PTFs applied:
# 7.3: MF63822
# 7.2: MF63833
# 7.1: MF63834
#
# Or you may hit this error during configure:
#
# Fatal: You must get working getaddrinfo() function.
#        or you can specify "--disable-ipv6".

# Currently SLIC is limited to supporting AT_FDCWD for dirfd on *at APIs :(
# Since Python internal code will use *at APIs when available, and expects them
# to support more than just AT_FDCWD, we have to disable them or you will just
# get lots of EINVAL returns.

%configure \
  CXX=g++ \
  CPPFLAGS="-I%{_includedir}/ncurses -D_ALL_SOURCE -D_XOPEN_SOURCE=700" \
  LDFLAGS="-Wl,-blibpath:%{_libdir}:/QOpenSys/usr/lib" \
  LDFLAGS_NODIST='-lutil' \
  ARFLAGS="-X${OBJECT_MODE} rc" \
  CCSHARED="-fPIC" \
  ac_cv_flock_decl=yes \
  ac_cv_func_flock=yes \
  ac_cv_lib_bsd_flock=no \
  ac_cv_func_clock_settime=no \
  ac_cv_func_posix_fadvise=no \
  ac_cv_func_posix_fallocate=no \
  ac_cv_func_sched_rr_get_interval=no \
  ac_cv_func_sched_setparam=no \
  ac_cv_func_sched_setscheduler=no \
  ac_cv_func_faccessat=no \
  ac_cv_func_fchmodat=no \
  ac_cv_func_fchownat=no \
  ac_cv_func_fstatat=no \
  ac_cv_func_futimesat=no \
  ac_cv_func_linkat=no \
  ac_cv_func_lstat=yes \
  ac_cv_func_mkdirat=no \
  ac_cv_func_mkfifoat=no \
  ac_cv_func_mknodat=no \
  ac_cv_func_openat=no \
  ac_cv_func_readlinkat=no \
  ac_cv_func_renameat=no \
  ac_cv_func_symlinkat=no \
  ac_cv_func_unlinkat=no \
  ac_cv_func_utimensat=no \
  --with-gcc \
  --with-threads \
  --with-system-expat \
  --with-system-ffi \
  --with-ensurepip=no \
  --with-tcltk-includes="$(pkg-config --cflags tk)" \
  --with-tcltk-libs="$(pkg-config --libs tk)" \
  --with-computed-gotos \
  --enable-ipv6 \
  --enable-loadable-sqlite-extensions \
  --enable-shared \
  #end

# Ensure -maix32/-maix64 is passed to gcc when used as the linker.
# TODO: This is hacky. Fix it properly
perl -p -i -e "s|ld_so_aix \$(CC)|ld_so_aix \$(CC) -maix${OBJECT_MODE}|" Makefile

perl -p -i -e 's|#! /usr/local/bin/python|#! %{_bindir}/python3|' Lib/cgi.py

%make_build

%install

%make_install

rm %{buildroot}/%{_bindir}/python3
rm %{buildroot}/%{_bindir}/2to3
rm %{buildroot}/%{_bindir}/idle3
rm %{buildroot}/%{_bindir}/pydoc3
rm %{buildroot}/%{_bindir}/pyvenv

%define site_packages "%{buildroot}/%{_libdir}/python%{major}.%{minor}/site-packages"

cp %{SOURCE1} %{site_packages}/sitecustomize.py

mv python-gdb.py %{buildroot}%{_libdir}/python3.6

%post
%{_bindir}/update-alternatives \
  --install %{_bindir}/python%{major} python%{major} %{_bindir}/python%{major}.%{minor} %{priority} \
  --slave %{_bindir}/2to3 2to3 %{_bindir}/2to3-%{major}.%{minor} \
  --slave %{_bindir}/idle3 idle3 %{_bindir}/idle%{major}.%{minor} \
  --slave %{_bindir}/pydoc3 pydoc3 %{_bindir}/pydoc%{major}.%{minor} \
  --slave %{_bindir}/pyvenv pyvenv %{_bindir}/pyvenv-%{major}.%{minor}

%{_bindir}/update-alternatives --install %{_bindir}/python python %{_bindir}/python%{major}.%{minor} %{priority}

%postun
%{_bindir}/update-alternatives --remove python%{major} %{_bindir}/python%{major}
%{_bindir}/update-alternatives --remove python %{_bindir}/python

%files
%defattr(-, qsys, *none)

%{_bindir}/2to3-%{major}.%{minor}
%{_bindir}/idle%{major}.%{minor}
%{_bindir}/pydoc%{major}.%{minor}
%{_bindir}/python%{major}.%{minor}
%{_bindir}/python%{major}.%{minor}m
%{_bindir}/pyvenv-%{major}.%{minor}

%ghost %{_bindir}/python3
%ghost %{_bindir}/2to3
%ghost %{_bindir}/idle3
%ghost %{_bindir}/pydoc3
%ghost %{_bindir}/pyvenv
%ghost %{_bindir}/python

%{_libdir}/libpython%{major}.%{minor}m.so

%{_libdir}/python%{major}.%{minor}

%if 0
Ensure that the build fails if required extensions are not built.

This will cause duplicate file build warnings, unfortunately :(
%endif
%{_libdir}/python%{major}.%{minor}/lib-dynload/_asyncio.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_bisect.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_blake2.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_bz2.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_cn.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_hk.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_iso2022.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_jp.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_kr.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_codecs_tw.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_crypt.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_csv.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_ctypes.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_ctypes_test.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_curses.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_curses_panel.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_datetime.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_dbm.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_decimal.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_elementtree.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_hashlib.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_heapq.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_json.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_lsprof.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_lzma.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_md5.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_multibytecodec.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_multiprocessing.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_opcode.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_pickle.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_posixsubprocess.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_random.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_sha1.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_sha256.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_sha3.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_sha512.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_socket.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_sqlite3.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_ssl.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_struct.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_testbuffer.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_testcapi.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_testimportmultiple.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/_testmultiphase.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/array.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/audioop.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/binascii.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/cmath.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/fcntl.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/grp.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/math.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/mmap.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/nis.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/parser.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/pyexpat.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/readline.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/resource.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/select.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/syslog.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/termios.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/unicodedata.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/xxlimited.so
%{_libdir}/python%{major}.%{minor}/lib-dynload/zlib.so

%exclude %{_libdir}/python%{major}.%{minor}/lib-dynload/_tkinter.so
%exclude %{_libdir}/python%{major}.%{minor}/tkinter
%exclude %{_libdir}/python%{major}.%{minor}/turtle.py
%exclude %{_libdir}/python%{major}.%{minor}/turtledemo

%{_mandir}/man1/*

%files devel
%defattr(-, qsys, *none)

%{_bindir}/python%{major}-config
%{_bindir}/python%{major}.%{minor}-config
%{_bindir}/python%{major}.%{minor}m-config

%{_includedir}/python%{major}.%{minor}m
%{_libdir}/pkgconfig/*
%{_libdir}/python%{major}.%{minor}/python-gdb.py

%files tkinter
%defattr(-, qsys, *none)

%{_libdir}/python%{major}.%{minor}/lib-dynload/_tkinter.so
%{_libdir}/python%{major}.%{minor}/tkinter
%{_libdir}/python%{major}.%{minor}/turtle.py
%{_libdir}/python%{major}.%{minor}/turtledemo


%changelog
* Thu Dec 16 2021 Abdirahim Musse <abdirahim.musse@ibm.com> - 3.6.15
- Update to 3.6.15

* Thu Nov 4 2021 Jason Hansen <stymie@us.ibm.com> - 3.6.12
- Intra-package Requires need version and release

* Thu Dec 10 2020 Naveen Ram <naveen.ram@ibm.com> - 3.6.12
- Ship python-gdb.py

* Tue Nov 10 2020 Abdirahim Musse<abdirahim.musse@ibm.com>
- Update to 3.6.12

* Thu Aug 13 2020 Kevin Adler <kadler@us.ibm.com>
- Disable symbol demangling when generating export files
- Newer PASE versions now demangle g++ symbols by default, breaking the
  build of matplotlib

* Tue Aug 11 2020 Xu Meng <mengxumx@cn.ibm.com> - 3.6.11
- Update to 3.6.11

* Fri Jul 17 2020 Abdirahim Musse <abdirahim.musse@ibm.com>
- Ghost python alternative

* Tue Jul 14 2020 Abdirahim Musse <abdirahim.musse@ibm.com>
- Fix dependency issues w/ update-alternatives

* Tue Jun 23 2020 Abdirahim Musse <abdirahim.musse@ibm.com>
- Move python*-config to devel package

* Tue Jun 23 2020 Abdirahim Musse <abdirahim.musse@ibm.com>
- Fix missing colon after Requires tag

* Fri Jun 19 2020 Abdirahim Musse <abdirahim.musse@ibm.com>
- Use update-alternatives

* Mon Jun 08 2020 Kevin Adler <kadler@us.ibm.com>
- Add libutil install requirement to match build requirement

* Mon Apr 27 2020 Kevin Adler <kadler@us.ibm.com> - 3.6.10
- Patch CVE-2019-18348 and CVE-2020-8492

* Fri Mar 13 2020 Kevin Adler <kadler@us.ibm.com>
- Enable computed gotos to improve bytecode interpreter performance
- See https://eli.thegreenplace.net/2012/07/12/computed-goto-for-efficient-dispatch-tables

* Wed Mar 11 2020 Kevin Adler <kadler@us.ibm.com>
- Fix errors building _csv and fcntl when using libutil 0.7
- Require libutil 0.7 for fix to work properly

* Tue Mar 3 2020 Xu Meng <mengxumx@cn.ibm.com> - 3.6.10
- Bump to 3.6.10
- Fix CVE-2019-16935
- Fix CVE-2019-16056

* Wed Oct 16 2019 Xu Meng <mengxumx@cn.ibm.com>
- Backport patch for CVE-2019-16935 from 3.6.10

* Tue Oct 01 2019 Mark Irish <mirish@ibm.com> - 3.6.9
- Bump to 3.6.9
- Fix CVE-2019-9948
- Fix CVE-2019-9947
- Fix CVE-2019-9740
- Fix CVE-2019-5010

* Tue Oct 01 2019 Mark Irish <mirish@ibm.com> - 3.6.8
- Added patches to fix issues with O_CLOEXEC and O_NOFOLLOW on pre-7.4 systems

* Mon Aug 26 2019 Xu Meng <mengxumx@cn.ibm.com>
- Add CFLAG '-fPIC' by default

* Wed Aug 07 2019 Mark Irish <mirish@ibm.com> - 3.6.8
- Update openssl-dev version to >= 1.1.1

* Fri Mar 15 2019 - Kevin Adler <kadler@us.ibm.com>
- Require libutil 0.6 (libutil.so.2)

* Mon Jan 14 2019 - Kevin Adler <kadler@us.ibm.com>
- Bump to 3.6.8
- Build with tcl/tk for tkinter package
- Speed up automatic dependency generation
- Fix invalid autoconf checks before the fact instead of after

* Sat Oct 20 2018 - Kevin Adler <kadler@us.ibm.com>
- Bump to 3.6.7

* Fri Aug 03 2018 - Kevin Adler <kadler@us.ibm.com>
- Don't set _LARGE_FILES on 64-bit, fixing pandas and scipy
- Prevent symbol duplication in python.exp on parallel builds

* Fri Jun 29 2018 - Jesse R. Gorzinski <jgorzins@us.ibm.com>
- Update to Python 3.6.6

* Thu Apr 26 2018 - Kevin Adler <kadler@us.ibm.com>
- Fix ncurses support, which gives us tab-completion and history in the REPL
- Prevent SIGILL from clock_getres on 7.2 and 7.3 systems withou PTFs

* Sat Mar 03 2018 - Jesse R. Gorzinski <jgorzins@us.ibm.com>
- Added patch for named semaphore problem

* Sat Mar 03 2018 - Kevin Adler <kadler@us.ibm.com>
- Add devel package

* Fri Feb 16 2018 - Kevin Adler <kadler@us.ibm.com>
- Build _locale, needed by YUM

* Tue Feb 13 2018 - Kevin Adler <kadler@us.ibm.com>
- Build bzip2 support, which yum requires

* Tue Feb 13 2018 - Kevin Adler <kadler@us.ibm.com>
- Ensure python builds with OpenSSL 1.0.2

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Build with expat, libffi, and ncurses

* Mon Feb 12 2018 - Kevin Adler <kadler@us.ibm.com>
- Define _POSIX_SOURCE and _ALL_SOURCE to add various POSIX features to Python,
  fixing an issue with yum running in C locale

* Wed Dec 20 2017 - Kevin Adler <kadler@us.ibm.com>
- Fix major(), minor(), and makedev() on 64-bit Python

* Wed Dec 20 2017 - Kevin Adler <kadler@us.ibm.com>
- Remove old 64-bit GCC script workarounds

* Wed Nov 22 2017 - Kevin Adler <kadler@us.ibm.com>
- Update BuildRequires for packages

* Tue Nov 21 2017 - Kevin Adler <kadler@us.ibm.com>
- Bump to 3.6.2
- Use system FFI library
- Fix shebang for cgi.py
- Provide Python abi needed for automatic python Requires

* Tue Nov 21 2017 - Kevin Adler <kadler@us.ibm.com>
- Remove manual Requires lines because we have automatic dependency generation

* Tue Nov 07 2017 - Kevin Adler <kadler@us.ibm.com>
- Fix licenses in spec files
